@@ -1,5 +1,23 @@
-"""$Id: fns.py,v 1.29 2006/11/23 10:02:48 rockyb Exp $
-Functions to support the Extended Python Debugger."""
+"""Functions to support the Extended Python Debugger.
+$Id: fns.py,v 1.42 2007/04/11 09:28:50 rockyb Exp $"""
+# -*- coding: utf-8 -*-
+#   Copyright (C) 2007 Rocky Bernstein
+#
+#    This program is free software; you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation; either version 2 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program; if not, write to the Free Software
+#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+#    02110-1301 USA.
+
 import inspect, linecache, os, shlex, sys, re, traceback, types
 
 # A pattern for a def header seems to be used a couple of times.
@@ -18,6 +36,14 @@ def arg_split(s,posix=False):
     lex.whitespace_split = True
     return list(lex)
 
+def count_frames(frame, count_start=0):
+    "Return a count of number of frames"
+    count = -count_start
+    while frame: 
+        count += 1
+        frame = frame.f_back
+    return count
+
 def decorate_fn_with_doc(new_fn, old_fn, additional_text=""):
     """Make new_fn have old_fn's doc string. This is particularly useful
     for the do_... commands that hook into the help system.
@@ -46,18 +72,27 @@ def checkline(obj, filename, lineno):
         return 0
     return lineno
 
+def file_pyc2py(filename):
+    """Given a file name, if the suffix is pyo or pyc (an optimized bytecode
+    file), change that to the py equivalent"""
+    if (filename.endswith(".pyc") or
+        filename.endswith(".pyo")):
+        return filename[:-1]
+    return filename
+
 def file2module(filename):
-    """Given a file name extract the most likely module name. """
+    """Given a file name, extract the most likely module name. """
     basename = os.path.basename(filename)
     if '.' in basename:
          pos = basename.rfind('.')
          return basename[:pos]
     else:
          return basename
+    return None
 
 def find_function(funcname, filename):
     try:
-        cre = re.compile(r'def\s+%s\s*[(]' % funcname)
+        cre = re.compile(r'def\s+%s\s*[(]' % re.escape(funcname))
         # cre = re.compile(r'%s\s*%s\s*[(]' % (_re_def_str, funcname))
     except:
         return None
@@ -86,7 +121,7 @@ def get_confirmation(obj, prompt, default=False):
     """
     while True and not obj.noninteractive:
         try:
-            reply = raw_input(prompt)
+            reply = raw_input(prompt).strip()
         except EOFError:
             reply = 'no'
             reply = reply.strip().lower()
@@ -141,15 +176,15 @@ def get_last_tb_or_frame_tb(frameno=1):
     fielding an exception. So assume that sys.exec_info()[frameno]
     is where we want to look."""
 
-    traceback = sys.exc_info()[frameno]
+    tb = sys.exc_info()[frameno]
 
     try:
         if inspect.istraceback(sys.last_traceback):
             # We do have a traceback so prefer that.
-            traceback = sys.last_traceback
+            tb = sys.last_traceback
     except AttributeError:
         pass
-    return traceback
+    return tb
 
 def print_dict(s, obj, title):
     if hasattr(obj, "__dict__"):
@@ -249,14 +284,14 @@ def print_stack_trace(obj, count=None):
             print_stack_entry(obj, i)
     except KeyboardInterrupt:
         pass
+    return
 
-def search_file(filename, path, cdir):
+def search_file(filename, directories, cdir):
     """Return a full pathname for filename if we can find one. path
     is a list of directories to prepend to filename. If no file is
-    found we'll return filename"""
+    found we'll return None"""
 
-    dirs=path.split(":")
-    for trydir in dirs:
+    for trydir in directories:
 
         # Handle $cwd and $cdir
         if trydir =='$cwd': trydir='.'
@@ -265,6 +300,7 @@ def search_file(filename, path, cdir):
         tryfile = os.path.abspath(os.path.join(trydir, filename))
         if os.path.isfile(tryfile):
             return tryfile
+    return None
     
 def show_onoff(bool):
     """Return 'on' for True and 'off' for False, and ?? for anything
@@ -273,8 +309,7 @@ def show_onoff(bool):
         return "??"
     if bool:
         return "on"
-    else:
-        return "off"
+    return "off"
 
 def parse_filepos(obj, arg):
     """parse_filepos(obj, arg)->(fn, filename, lineno)
@@ -345,15 +380,26 @@ def get_brkpt_lineno(obj, arg):
             lineno = int(ln)
     return (funcname, filename, lineno)
 
+def whence_file(py_script):
+    """Do a shell-like path lookup for py_script and return the results.
+    If we can't find anything return py_script"""
+    if py_script.find(os.sep) != -1:
+        # Don't search since this name has path separator components
+        return py_script
+    for dirname in os.environ['PATH'].split(os.pathsep):
+        py_script_try = os.path.join(dirname, py_script)
+        if os.path.exists(py_script_try):
+            return py_script_try
+    # Failure
+    return py_script
 
 if __name__ == '__main__':
     print "show_onoff(True is %s)" % str(show_onoff(True))
     assert show_onoff(True) == 'on'
     print "show_onoff(False is %s)" % str(show_onoff(False))
     assert show_onoff(False) == 'off'
-    print "search_file('fns.py', '.', '.'): %s" % search_file("fns.py",
-                                                              "$cwd:$cdir",
-                                                              ".")
+    print "search_file('fns.py', '.', '.'): %s" % (
+        search_file("fns.py", ["$cwd", "$cdir"], "."))
     assert printf(31, "/o") == '037'
     assert printf(31, "/t") == '00011111'
     assert printf(33, "/c") == '!'
