@@ -1,29 +1,50 @@
-"""$Id: gdb.py.in,v 1.98 2006/12/10 13:26:24 rockyb Exp $
-Handles gdb-like command processing.
-(See also pydb.doc for documentation.)"""
+"""Handles gdb-like command processing.
+(See also pydb.doc for documentation.)
+$Id: gdb.py.in,v 1.131 2007/04/07 10:53:28 rockyb Exp $"""
+# -*- coding: utf-8 -*-
+#   Copyright (C) 2007 Rocky Bernstein
+#
+#    This program is free software; you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation; either version 2 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program; if not, write to the Free Software
+#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+#    02110-1301 USA.
 
 _debugger_name = 'pydb'
 
 # The name of the debugger we are currently going by.
-_version = '1.20'
+_version = '1.22'
+
+import atexit, inspect, linecache, os, pprint, pydoc, shlex
+import bdb, disassemble, re, subcmd, sys, types
 
-import atexit, dis, inspect, linecache, os, pprint
-import bdb, re, subcmd, sys, types
 
 import pydbcmd, pydbbdb
 import signal
 import sighandler
+from complete import *
 
 ## from connection import ConnectionServerFactory, ConnectionClientFactory, ConnectionFailed
-from display    import Display, DisplayNode
-from fns        import *
-from info       import *
-from set        import *
-from show       import *
+from bdb         import BdbQuit
+from display     import Display, DisplayNode
+from fns         import *
+from info        import *
+from set         import *
+from show        import *
 
 from pydbcmd import Cmd
 from pydbbdb import Bdb
 
+from threadinfo import *
 
 class Restart(Exception):
     """Causes a debugger to be restarted for the debugged Python program."""
@@ -61,6 +82,8 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.__init_set()    # Initialize 'set' subcommands
         self.__init_show()   # Initialize 'show' subcommands
 
+        self.all_completions = lambda arg, left=True: \
+                               all_completions(self, arg, left)
         self.basename        = False
         self.connection      = None   # For out-of-process communication
         self.currentbp       = None   # for "info program"
@@ -68,7 +91,18 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.dbg_pydb        = False  # Debug the debugger?
         self.debug_signal    = None   # The signal used by 'attach'
         self.display         = Display()
+        self.fntrace         = False  # Tracing functions/methods
         self.gdb_dialect     = True   # Controls how stack is shown
+        self.field_BdbQuit   = False  # does dispatcher field BdbQuit?
+
+        # A list of the commands for which the first argument can be a
+        # Python object. This is used by subcommand completions.
+        self.first_can_be_obj = [
+            "disassemble", "examine", "help", "p", "pp", "pydoc", "x",
+            "whatis"
+            ]
+        
+        self.is_in_dbg       = is_in_gdb
         self.lastcmd         = ''     # last debugger command run
         self.linetrace       = False
         self.linetrace_delay = 0
@@ -87,7 +121,7 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.orig_stdin      = self.stdin
         self.orig_stdout     = self.stdout
         self.running         = False
-        self.search_path     = '$cdir:$cwd'  # source name search path
+        self.search_path     = sys.path  # source name search path
         self.sigcheck        = True
         self.stop_reason     = None    # Why are we in the debugger?
         self._sys_argv       = []      # exec sys.argv, e.g. may include pydb
@@ -140,6 +174,9 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         # Try to load readline if it exists
         try:
             import readline
+            import rlcompleter
+            self.readline  = readline
+            self.completer = rlcompleter.Completer(sys._getframe().f_locals)
             self.hist_last = 0
             self.hist_save = False
             self.set_history_length = readline.set_history_length
@@ -170,9 +207,9 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         return
 
     def break_anywhere_gdb(self, frame):
-        """Decorate bdb break_anyhwere to consider the sigcheck and
+        """Decorate bdb break_anywhere to consider the sigcheck and
         linetrace flags."""
-        if self.sigcheck or self.linetrace: return True
+        if self.sigcheck or self.linetrace or self.fntrace: return True
         return self.break_anywhere(frame)
 
     ### FIXME: trace_dispatch could be a performance bottleneck. If
@@ -182,10 +219,34 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
     def trace_dispatch_gdb(self, frame, event, arg):
         """Check to see if the signal handler's we are interested have
         changed. If so we'll intercept them. """
-        if not hasattr(self, 'thread_name') \
-           or self.thread_name == 'MainThread':
+        if (not hasattr(self, 'thread_name') 
+            or self.thread_name == 'MainThread'):
             self.sigmgr.check_and_adjust_sighandlers()
-        return self.trace_dispatch_old(frame, event, arg)
+
+        # The below variable will be used to scan down frames to determine
+        # if trace_dispatch has been called. We key on the variable
+        # name, method name, type of variable and even the value.
+
+        breadcrumb = is_in_gdb_dispatch
+
+        try:
+            return self.trace_dispatch_old(frame, event, arg)
+        except BdbQuit:
+            if self.field_BdbQuit:
+                # If we have one thread (and know that this this is the case),
+                # then we can use sys.exit(). Otherwise kill is the
+                # only way to leave.
+                if hasattr(sys, '_current_frames'):
+                    if len(sys._current_frames()) == 1:
+                        sys.exit()
+                if self.noninteractive:
+                    arg='unconditionally'
+                else:
+                    arg=''
+                self.do_kill('')
+                return False
+            else:
+                raise BdbQuit
 
     def __adjust_frame(self, pos, absolute_pos):
         """Adjust stack frame by pos positions. If absolute_pos then
@@ -206,17 +267,17 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
                 if pos >= 0:
                     pos = len(self.stack)-pos-1
                 else:
-                    pos -= len(self.stack)
+                    pos = -pos-1
             elif pos < 0:
                 pos = len(self.stack)+pos
         else:
             pos += self.curindex
 
         if pos < 0:
-            self.errmsg("Adjusting would be put us beyond the oldest frame")
+            self.errmsg("Adjusting would put us beyond the oldest frame")
             return
         elif pos >= len(self.stack):
-            self.errmsg("Adjusting would be put us beyond the newest frame")
+            self.errmsg("Adjusting would put us beyond the newest frame")
             return
 
         self.curindex = pos
@@ -224,49 +285,6 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.print_location()
         self.lineno = None
 
-    def __complete_subcommand(self, subcmd, text):
-        """Print a list of completions for subcmd that start with text.
-           We get the list of completions from self._*subcmd*_cmds.
-           If no completion we return the empty list.
-           """
-        matches = []
-        subcmd_name = "%scmds" % subcmd[0]
-        if hasattr(self, subcmd_name):
-            subcmd_obj = getattr(self, subcmd_name)
-            if hasattr(subcmd_obj, "cmdlist"):
-                for word in subcmd_obj.cmdlist:
-                    if word.startswith(text):
-                        matches.append("%s %s" % (subcmd[0], word))
-        return matches
-
-    def all_completions(self, arg):
-        """Return a list of command names that can start with the
-        supplied command prefix."""
-        if not arg:
-            cmd_prefix=''
-            args=[]
-        else:
-            args = arg.split()
-            cmd_prefix=args[0]
-        if len(args) > 1:
-            complete_cmds = pydbcmd.Cmd.complete(self, cmd_prefix, 0)
-            if complete_cmds is not None and args[0] in complete_cmds:
-                return self.__complete_subcommand(args, args[1])
-            return
-
-        seen = {};
-        i=0
-        completions = []
-        text=pydbcmd.Cmd.complete(self, cmd_prefix, i)
-        while True:
-            if text is None: break
-            if text not in seen.keys():
-                completions.append(text)
-                seen[text]=True
-            i += 1
-            text=pydbcmd.Cmd.complete(self, cmd_prefix, i)
-        return completions
-
     def __do_complete(self, arg):
         """Print a list of command names that can start with the
         supplied command prefix. If there is no completion nothing
@@ -291,7 +309,25 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.infocmds.add('program',        self.info_program)
         self.infocmds.add('signal',         self.sigmgr.info_signal, 2, False)
         self.infocmds.add('source',         self.info_source, 2)
+
+        if hasattr(sys, "_current_frames"):
+            self.info_threads = lambda arg: info_thread_new(self, arg)
+            doc               = info_thread_new.__doc__
+        else:
+            try:
+                import threadframe
+                sys._current_frames = threadframe.dict # Make it look like 2.5
+                self.info_threads   = lambda arg: info_threadframe(self, arg)
+                doc                 = info_threadframe.__doc__
+                
+            except:
+                self.info_threads = lambda arg: info_thread_old(self, arg)
+                doc               = info_thread_old.__doc__
+        self.info_threads.__doc__ = doc
+
+        self.infocmds.add('threads', self.info_threads, 2)
         ## self.infocmds.add('target',         self.info_target)
+        return
 
     def __init_set(self):
         """Initialize set subcommands. Note: instance variable name
@@ -301,8 +337,9 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.setcmds=subcmd.Subcmd("set",   self.help_set.__doc__)
         self.setcmds.add('args',            self.set_args)
         self.setcmds.add('basename',        self.set_basename)
-        self.setcmds.add('cmdtrace',        self.set_cmdtrace)
         self.setcmds.add('debug-pydb',      self.set_dbg_pydb)
+        self.setcmds.add('flush',           self.set_flush)
+        self.setcmds.add('fntrace',         self.set_fntrace, 2)
         ## self.setcmds.add('debug-signal',    self.set_debug_signal)
         self.setcmds.add('history',         self.set_history)
         self.setcmds.add('interactive',     self.set_interactive)
@@ -313,6 +350,7 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.setcmds.add('prompt',          self.set_prompt)
         self.setcmds.add('sigcheck',        self.set_sigcheck)
         ## self.setcmds.add('target-address',  self.set_target_address)
+        self.setcmds.add('trace-commands',  self.set_cmdtrace)
         self.setcmds.add('warnoptions',     self.set_warnoptions)
         return
 
@@ -325,9 +363,10 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.showcmds.add('basename',       self.show_basename)
         self.showcmds.add('debug-pydb',     self.show_dbg_pydb)
         ## self.showcmds.add('debug-signal',   self.show_debug_signal)
-        self.showcmds.add('cmdtrace',       self.show_cmdtrace,    2)
         self.showcmds.add('commands',       self.show_commands,    2, False)
         self.showcmds.add('directories',    self.show_directories, 1)
+        self.showcmds.add('fntrace',        self.show_fntrace,     2)
+        self.showcmds.add('flush',          self.show_flush)
         self.showcmds.add('history',        self.show_history)
         self.showcmds.add('interactive',    self.show_interactive)
         self.showcmds.add('linetrace',      self.show_linetrace,   3)
@@ -337,6 +376,7 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.showcmds.add('prompt',         self.show_prompt)
         self.showcmds.add('sigcheck',       self.show_sigcheck)
         ## self.showcmds.add('target-address', self.show_target_address)
+        self.showcmds.add('trace-commands', self.show_cmdtrace,    1)
         self.showcmds.add('version',        self.show_version)
         self.showcmds.add('warnoptions',    self.show_warnoptions)
 
@@ -353,12 +393,9 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
     def display_enable(self, arg, flag):
         # arg is list of display points
         for i in arg:
-            try:
-                i = int(i)
+            i = self.get_an_int(i, 'index %r is not a number' % i)
+            if i is not None:
                 self.display.enable(i, flag)
-            except ValueError:
-                self.msg('index %r is not a number' % i)
-                continue
         return
 
     def lineinfo(self, identifier):
@@ -401,12 +438,12 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.curframe = None
         return
 
-    def interaction(self, frame, traceback):
+    def interaction(self, frame, tb):
         """Possibly goes into loop to read debugger commands."""
-        do_loop = self.setup(frame, traceback) != 1
+        do_loop = self.setup(frame, tb) != 1
         if do_loop:
-            if frame:
-                self.print_location(print_line=self.linetrace)
+            if frame or tb:
+                self.print_location(print_line=self.linetrace or self.fntrace)
                 self.display.displayAny(self.curframe)
             if not self.noninteractive:
                 try:
@@ -451,15 +488,15 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
                         self.errmsg('filename in the range must be the same')
                         return (None, None, None)
                     if last < first:
-                        try:
-                            int(args[1])
-                            # Assume it's a count
-                            last = first + last
-                        except ValueError:
-                            self.errmsg('2nd number (%s) is less than the '
-                                        + '1st number %d is not a pure integer'
-                                        % args[1], first)
+                        i = self.get_an_int(args[1],
+                                            ('2nd number (%s) is less than the'
+                                            + ' 1st number %d, but it does not'
+                                            + ' evaluate to an integer')
+                                            % (args[1], first))
+                        if i is None:
                             return (None, None, None)
+                        # Assume last is a count rather than an end line number
+                        last = first + last
                 elif len(args) > 2:
                     self.errmsg('At most 2 arguments allowed. Saw: %d' %
                                 len(args))
@@ -481,7 +518,7 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.quitting = 0
         return
 
-    def setup(self, frame, traceback=None):
+    def setup(self, frame, tb=None):
         """Initialization done before entering the debugger-command
         loop. In particular we set up the call stack used for local
         variable lookup and frame/up/down commands.
@@ -491,8 +528,8 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.forget()
         if self.dbg_pydb:
             frame = inspect.currentframe()
-        if frame:
-            self.stack, self.curindex = self.get_stack(frame, traceback)
+        if frame or tb:
+            self.stack, self.curindex = self.get_stack(frame, tb)
             self.curframe = self.stack[self.curindex][0]
         else:
             self.stack = self.curframe = self.botframe = None
@@ -635,11 +672,13 @@ alias ps pi self
             self.aliases[args[0]] = ' '.join(args[1:])
 
     def do_break(self, arg, temporary = 0, thread_name=None):
+
         """b(reak) [[file:]lineno | function] [, condition]
-With a line number argument, set a break there in the current
-file.  With a function name, set a break at first executable line
-of that function.  Without argument, list all breaks.  If a second
-argument is present, it is a string specifying an expression
+
+With a line number argument, set a break there in the current file.
+With a function name, set a break at first executable line of that
+function.  Without argument, set a breakpoint at current location.  If
+a second argument is present, it is a string specifying an expression
 which must evaluate to true before the breakpoint is honored.
 
 The line number may be prefixed with a filename and a colon,
@@ -779,7 +818,7 @@ the .py suffix may be omitted."""
             bnum = len(bdb.Breakpoint.bpbynumber)-1
         else:
             try:
-                bnum = self.get_pos_int(arg, min=0, default=None,
+                bnum = self.get_pos_int(arg, min_value=0, default=None,
                                         cmdname="'commands'")
             except ValueError:
                 return False
@@ -818,7 +857,7 @@ the .py suffix may be omitted."""
 
         args = arg.split(' ', 1)
         try:
-            bpnum = self.get_pos_int(args[0].strip(), min=1,
+            bpnum = self.get_pos_int(args[0].strip(), min_value=1,
                                      cmdname='condition')
         except ValueError:
             return
@@ -831,7 +870,7 @@ the .py suffix may be omitted."""
         try:
             bp = bdb.Breakpoint.bpbynumber[bpnum]
         except IndexError:
-            self.errmsg("No breakpoint number %d." % bpnum)
+            self.errmsg("No breakpoint numbered %d." % bpnum)
             return False
         if bp:
             bp.cond = cond
@@ -849,7 +888,7 @@ line position is given, continue until that line is reached. This is
 exactly the same thing as setting a temporary breakpoint at that
 position before running an (unconditional) continue."""
         if not self.is_running(): return
-        if self.linetrace:
+        if self.linetrace or self.fntrace:
             # linetracing is like stepping, but we just don't stop. If
             # we were to calling set_continue, it  *might* remove all
             # stopping if there were no breakpoints.
@@ -858,13 +897,16 @@ position before running an (unconditional) continue."""
             self.stepping = True
         else:
             self.stepping = False
-            self.set_continue()
 
         if arg:
             (funcname, filename, lineno) = parse_filepos(self, arg)
             if lineno:
                 self.do_tbreak(arg)
 
+        # This has to be done after do_tbreak
+        if not (self.linetrace or self.fntrace):
+            self.set_continue()
+            
         # Tell cmdloop to break out of its loop.
         return True
 
@@ -889,6 +931,7 @@ position before running an (unconditional) continue."""
         p.aliases         = self.aliases
         p.basename        = self.basename
         p.cmdtrace        = self.cmdtrace
+        p.fntrace         = self.fntrace
         p.gdb_dialect     = self.gdb_dialect
         p.infocmds        = self.infocmds
         p.linetrace       = self.linetrace
@@ -932,7 +975,7 @@ position before running an (unconditional) continue."""
         numberlist = arg.split()
         for arg in numberlist:
             try:
-                i = self.get_pos_int(arg, min=1, default=None,
+                i = self.get_pos_int(arg, min_value=1, default=None,
                                      cmdname='delete')
             except ValueError:
                 continue
@@ -958,11 +1001,11 @@ the default."""
         if len(args) == 0:
             if get_confirmation(self,
                 'Reinitialize source path to empty? (y or n) '):
-                self.search_path=''
+                self.search_path=[]
             return
         else:
             # FIXME: Loop over arguments checking for directory?
-            self.search_path=args[0]
+            self.search_path.insert(0, args[0])
 
     def do_disable(self, arg):
         """disable [display] bpnumber [bpnumber ...]
@@ -972,7 +1015,7 @@ the default."""
 
         args = arg.split()
         if len(args) == 0:
-            self.errmsg('No breakpoint number given')
+            self.errmsg('No breakpoint number given.')
             return
         if args[0] == 'display':
             self.display_enable(args[1:], 0)
@@ -981,11 +1024,11 @@ the default."""
             try:
                 i = int(i)
             except ValueError:
-                self.msg('Breakpoint index %r is not a number' % i)
+                self.msg('Breakpoint index %r is not a number.' % i)
                 continue
 
             if not (0 <= i < len(bdb.Breakpoint.bpbynumber)):
-                self.msg('No breakpoint numbered %d' % i)
+                self.msg('No breakpoint numbered %d.' % i)
                 continue
 
             bp = bdb.Breakpoint.bpbynumber[i]
@@ -993,38 +1036,65 @@ the default."""
                 bp.disable()
 
     def do_disassemble(self, arg):
-        """disassemble [location]
-        With no argument disassemble at the current frame location.
-        With a numeric argument, disassemble at the frame location at that
-        line number. With a class, method, function, code or string argument
-        disassemble that."""
-        if not self.curframe:
-            self.msg("No frame selected.")
-            return
+        """disassemble [obj-or-class] [start line number [end line number]]
 
-        # Disassemble uses "print". We may have redirected where we want
-        # debugger output to go. So we'll save and restore sys.stdout.
-        # Observation and fix from Matt Flemming.
-        orig_stdout = sys.stdout
-        sys.stdout = self.stdout
+With no argument, disassemble the current frame.  With one integer
+argument, the disassembly is narrowed to not show lines before that
+line number. With a second integer argument, disassembly ends as soon
+a the the line number exceeds the ending line number.
+
+With a class, method, function, code or string argument disassemble
+that."""
+
+        if arg:
+            args = arg.split()
+        else:
+            args = []
+        start_line = end_line = None
         try:
-            if not arg:
-                dis.disassemble(self.curframe.f_code, self.curframe.f_lasti)
-            else:
-                try:
-                    i = int(arg)
-                    dis.disassemble(self.curframe.f_code, i)
-                except ValueError:
+            if len(args) > 0:
+                start_line = int(args[0])
+
+                if len(args) == 2:
                     try:
-                        obj=self.getval(arg)
-                        dis.dis(obj)
-                    except NameError:
-                        pass
-        except:
-            # Make sure we restore sys.stdout on an exception.
-            sys.stdout = orig_stdout
-            raise # reraise exception
-        sys.stdout = orig_stdout
+                        end_line = self.get_int(args[1], cmdname="disassemble")
+                    except ValueError:
+                        return False
+                elif len(args) > 2:
+                    self.errmsg("Expecting 1-2 line parameters, got %d" %
+                                len(args))
+                    return False
+            if not self.curframe:
+                self.msg("No frame selected.")
+                return False
+            disassemble.disassemble(self, self.curframe.f_code,
+                                    start_line=start_line,
+                                    end_line=end_line)
+        except ValueError:
+            try:
+                if len(args) > 1:
+                    try:
+                        start_line = self.get_int(args[1],
+                                                  cmdname="disassemble")
+                        if len(args) == 3:
+                            end_line = self.get_int(args[2],
+                                                    cmdname="disassemble")
+                        elif len(args) > 3:
+                            self.errmsg("Expecting 0-3 parameters, got %d" %
+                                        len(args))
+                            return False
+                    except ValueError:
+                        return False
+
+                if hasattr(self, 'curframe') and self.curframe:
+                    obj=self.getval(args[0])
+                else:
+                    obj=eval(args[0])
+                disassemble.dis(self, obj,
+                                start_line=start_line, end_line=end_line)
+            except NameError:
+                self.errmsg("Object '%s' is not something we can disassemble"
+                            % args[0])
         return False
 
     def do_display(self, arg):
@@ -1088,14 +1158,11 @@ the default."""
             return
 
         for i in args:
-            try:
-                i = int(i)
-            except ValueError:
-                self.msg('Breakpoint index %r is not a number' % i)
-                continue
+            i = self.get_an_int(i, 'Breakpoint index %r is not a number' % i)
+            if i is None: continue
 
-            if not (0 <= i < len(bdb.Breakpoint.bpbynumber)):
-                self.errmsg('No breakpoint numbered %d' % i)
+            if not (1 <= i < len(bdb.Breakpoint.bpbynumber)):
+                self.errmsg('No breakpoint numbered %d.' % i)
                 continue
 
             bp = bdb.Breakpoint.bpbynumber[i]
@@ -1127,7 +1194,7 @@ is no Python file."""
                                    % self.mainpyfile):
                    # XXX how to clean up name space ??
                    self._program_sys_argv = None
-                   self.mainpyfile == ""
+                   self.mainpyfile = ""
         else:
             # XXX how to clean up name space ??
             self.mainpyfile=fname
@@ -1165,13 +1232,11 @@ is no Python file."""
 
         if not self.stack:
             self.msg("Program has no stack frame set.")
-            return
-        try:
-            arg = int(arg)
-        except ValueError:
-            self.errmsg(("The 'frame' command requires a frame number."+
-                         " Got: %s") % arg)
-            return
+            return False
+        arg = self.get_an_int(arg,
+                              ("The 'frame' command requires a frame number."+
+                               " Got: %s") % arg)
+        if arg is None: return False
 
         i_stack = len(self.stack)
         if arg < -i_stack or arg > i_stack-1:
@@ -1179,6 +1244,7 @@ is no Python file."""
                   % (-i_stack, i_stack-1))
         else:
             self.__adjust_frame(pos=arg, absolute_pos=True)
+        return False
 
     def do_handle(self, arg):
         """Specify how to handle a signal.
@@ -1205,11 +1271,23 @@ recognized actions include "stop", "nostop", "print", "noprint",
         reached and the breakpoint is not disabled and any associated
         condition evaluates to true."""
         args = arg.split()
-        bpnum = int(args[0].strip())
-        try:
-            count = int(args[1].strip())
-        except:
+
+        bpnum = self.get_an_int(args[0],
+                                ("ignore: bpnumber %s does not evaluate to an"
+                                + " integer") % args[0], min_value=1)
+        if bpnum is None: return False
+        if not (0 <= bpnum < len(bdb.Breakpoint.bpbynumber)):
+                self.errmsg('No breakpoint numbered %d.' % bpnum)
+                return False
+
+        if len(args) > 1:
+            count = self.get_an_int(args[1],
+                                    ("ignore: count %s does not evaluate to an"
+                                    + " integer.") % args[0], min_value=0)
+            if count is None: return False
+        else:
             count = 0
+            
         bp = bdb.Breakpoint.bpbynumber[bpnum]
         if bp:
             bp.ignore = count
@@ -1249,24 +1327,24 @@ recognized actions include "stop", "nostop", "print", "noprint",
         Set the next line that will be executed."""
 
         if not self.is_running():
-            return
+            return False
 
         if self.curindex + 1 != len(self.stack):
             self.errmsg("You can only jump within the bottom frame")
-            return
+            return False
+        arg = self.get_an_int(arg,
+                              "jump: a line number is required, got %s." %
+                              arg)
+        if arg is None: return False
         try:
-            arg = int(arg)
-        except ValueError:
-            self.errmsg("The 'jump' command requires a line number.")
-        else:
-            try:
-                # Do the jump, fix up our copy of the stack, and display the
-                # new position
-                self.curframe.f_lineno = arg
-                self.stack[self.curindex] = self.stack[self.curindex][0], arg
-                self.print_location()
-            except ValueError, e:
-                self.errmsg('%s failed: %s' % (cmdname, e))
+            # Do the jump, fix up our copy of the stack, and display the
+            # new position
+            self.curframe.f_lineno = arg
+            self.stack[self.curindex] = self.stack[self.curindex][0], arg
+            self.print_location()
+        except ValueError, e:
+            self.errmsg('%s failed: %s' % (cmdname, e))
+        return False
 
     def do_kill(self, arg):
         """kill [unconditionally]
@@ -1361,6 +1439,24 @@ With an integer argument, perform that many times."""
         "Print working directory."
         self.msg('Working directory ' + os.getcwd() + '.')
 
+    def do_pydoc(self, arg):
+        """pydoc <name> ...
+
+Show pydoc documentation on something. <name> may be the name of a
+Python keyword, topic, function, module, or package, or a dotted
+reference to a class or function within a module or module in a
+package.  If <name> contains a '/', it is used as the path to a Python
+source file to document. If name is 'keywords', 'topics', or
+'modules', a listing of these things is displayed.
+"""
+        sys_path_save = list(sys.path)
+        sys_argv_save = list(sys.argv)
+        sys.argv      = ['pydoc'] + shlex.split(arg)
+        pydoc.cli()
+        sys.argv      = sys_argv_save
+        sys.path      = sys_path_save
+        return False
+
     def do_quit(self, arg):
         """q(uit) or exit - Quit the debugger.  The program being
         executed is aborted."""
@@ -1394,10 +1490,12 @@ With an integer argument, perform that many times."""
             self.do_rquit(None)
         else:
             if self._sys_argv[0]:
-                self.msg("Re exec'ing\n\t%s" % self._sys_argv)
+                self.msg("Re exec'ing:\n\t%s" % self._sys_argv)
                 os.execvp(self._sys_argv[0], self._sys_argv)
             else:
                 self.msg("No exectuable file specified.")
+        return False
+        
 
     def do_return(self, arg):
         """Make selected stack frame return to its caller. Control
@@ -1415,7 +1513,7 @@ With an integer argument, perform that many times."""
         while True and not self.noninteractive:
             try:
                 # reply = raw_input('Make %s return now? (y or n) ')
-                reply = raw_input('Return now? (y or n) ')
+                reply = raw_input('Return now? (y or n) ').strip()
             except EOFError:
                 reply = 'no'
                 reply = reply.strip().lower()
@@ -1453,6 +1551,7 @@ With an integer argument, perform that many times."""
 
         # print "++i: %d, last_stmt %d, line: %d " % (i, last_stmt, last_line)
         self.do_jump(last_line, "Return")
+        return False
 
     def do_retval(self, arg):
         """retval - Show the value that is to be returned from a
@@ -1463,6 +1562,8 @@ assignment to the variable __return__."""
             self.msg(self.curframe.f_locals['__return__'])
         else:
             self.errmsg('Not yet returned!')
+        return False
+
     do_rv = do_retval
 
     def do_run(self, arg_str):
@@ -1494,10 +1595,12 @@ assignment to the variable __return__."""
             self.errmsg("Expecting at least 2 arguments, got %d." % len(args))
             return
         self.setcmds.do(self, args[0], args)
+        return False
 
     def do_shell(self, arg):
         """Execute the rest of the line as a shell command."""
         os.system(arg)
+        return False
 
     def do_show(self, arg):
 
@@ -1512,7 +1615,7 @@ assignment to the variable __return__."""
                 if self.showcmds.subcmds[subcommand]['in_list']:
                     self.msg_nocr("%s: " % subcommand)
                     self.do_show(subcommand)
-            return
+            return False
 
         if self._re_linetrace_delay.match(arg):
             self.msg("line trace delay is %s. (In seconds)"
@@ -1521,6 +1624,8 @@ assignment to the variable __return__."""
             args = arg.split()
             self.showcmds.do(self, args[0], args)
 
+        return False
+            
     def do_signal(self, arg):
         """signal [signum|signame]
 
@@ -1530,7 +1635,7 @@ Send a signal to the debugged process.
             signum = 0
         else:
             try: 
-                signum = int(arg)
+                signum = int(eval(arg))
                 if sighandler.lookup_signame(signum) == None:
                     self.msg("Signal number %d not a known signal number."
                              % signum)
@@ -1542,7 +1647,7 @@ Send a signal to the debugged process.
                              % arg)
                     return
         os.kill(os.getpid(), signum)
-
+        return False
         
     def do_source(self, arg):
         """source FILE
@@ -1554,7 +1659,8 @@ Send a signal to the debugged process.
         file and control is returned to the console."""
         self.setup_source(os.path.expanduser(arg), True);
         rc = self.execRcLines()
-        if rc == 1:  return rc
+        if rc == 1:  return True
+        return False
 
     def do_step(self, arg):
         """s(tep) [count]
@@ -1591,6 +1697,7 @@ Deletes the specified alias."""
         if len(args) == 0: return
         if args[0] in self.aliases:
             del self.aliases[args[0]]
+        return False
 
     # Print a traceback starting at the top stack frame.
     # The most recently entered frame is printed last;
@@ -1610,13 +1717,12 @@ Deletes the specified alias."""
         if arg:
             args = arg.split()
             for i in args:
-                try:
-                    i = int(i)
+                i = self.get_an_int(i, 'index %r is not a number' % i)
+                if i is not None:
                     self.display.deleteOne(i)
-                except ValueError:
-                    self.errmsg('index %r is not a number' % i)
         else:
             self.display.deleteAll()
+        return False
 
     def do_up(self, arg):
         """up [count]
@@ -1634,6 +1740,7 @@ Deletes the specified alias."""
         if self.gdb_dialect:
             count = -count
         self.__adjust_frame(pos=count, absolute_pos=False)
+        return False
 
     def do_whatis(self, arg):
         """whatis arg
@@ -1788,9 +1895,6 @@ was reached.
             return
         return
 
-    def help_pydb(self, *arg):
-        help()
-
     # Note: format of help is compatible with ddd.
     def help_set(self, args):
         """This command modifies parts of the debugger environment.
