@@ -1,8 +1,28 @@
-"""$Id: info.py,v 1.5 2006/09/03 11:41:48 rockyb Exp $
-show subcommands, except those that need some sort of text substitution.
+"""'show' subcommands, except those that need some sort of text substitution.
 (Those are in gdb.py.in.)
 """
-import bdb, fns, inspect, os, pprint
+__revision = "$Id: info.py,v 1.12 2007/02/14 12:10:03 rockyb Exp $"
+# -*- coding: utf-8 -*-
+#   Copyright (C) 2006, 2007 Rocky Bernstein
+#
+#    This program is free software; you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation; either version 2 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program; if not, write to the Free Software
+#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+#    02110-1301 USA.
+
+import bdb, fns, inspect, os, pprint, sys
+
+# from threadinfo import *
 
 class SubcmdInfo:
 
@@ -30,6 +50,7 @@ class SubcmdInfo:
             self.msg_nocr("%s=" %  name)
             if name in d: self.msg(d[name])
             else: self.msg("*** undefined ***")
+        return False
 
     def info_breakpoints(self, arg):
         """Status of user-settable breakpoints.
@@ -44,11 +65,13 @@ The short command name is L."""
                     self.bpprint(bp)
         else:
             self.msg("No breakpoints.")
+        return False
 
     def info_display(self, arg):
         """Expressions to display when program stops, with code numbers."""
         if not self.display.displayAll():
             self.msg('There are no auto-display expressions now.')
+        return False
 
     def info_globals(self, arg):
         """Global variables of current stack frame"""
@@ -58,6 +81,7 @@ The short command name is L."""
         self.msg("\n".join(["%s = %s"
                             % (l, pprint.pformat(self.getval(l)))
                             for l in self.curframe.f_globals]))
+        return False
 
     def info_line(self, arg):
         """Current line number in source file"""
@@ -71,7 +95,8 @@ The short command name is L."""
             if answer[0]:
                 item, filename, lineno = answer
                 if not os.path.isfile(filename):
-                    filename = fns.search_file(filename, self.search_path,
+                    filename = fns.search_file(filename,
+                                               self.search_path,
                                                self.main_dirname)
                 self.msg('Line %s of "%s" <%s>' %
                          (lineno, filename, item))
@@ -85,6 +110,7 @@ The short command name is L."""
                  (inspect.getlineno(self.curframe),
                   self.filename(self.canonic_filename(self.curframe)),
                   self.curframe.f_lasti))
+        return False
 
     def info_locals(self, arg):
         """Local variables of current stack frame"""
@@ -111,7 +137,10 @@ The short command name is L."""
             elif self.stop_reason == 'return':
                 self.msg('It stopped at a return.')
             else:
-                self.msg("It stopped after stepping, next'ing or initial start.")
+                self.msg("It stopped after stepping, next'ing " +
+                         "or initial start.")
+        return False
+    
     def info_source(self, arg):
         """Information about the current Python file."""
         if not self.curframe:
@@ -119,8 +148,10 @@ The short command name is L."""
             return
         self.msg('Current Python file is %s' %
                  self.filename(self.canonic_filename(self.curframe)))
+        return False
 
     def info_target(self, args):
         """Display information about the current target."""
         self.msg('target is %s' % self.target)
+        return False
 
