@@ -1,17 +1,18 @@
 #!/usr/local/bin/python -t
-"""$Id: pydb.py.in,v 1.132 2006/12/10 13:26:24 rockyb Exp $
-A Python debugger. Contains user-callable routines, e.g.
+"""A Python debugger. Contains user-callable routines, e.g.
 exception_hook, run, runl, runv, debugger (AKA set_trace), post_mortem,
 or runevel.
 
-(See also pydb.doc for documentation.)"""
+(See also pydb.doc for documentation.)
+
+$Id: pydb.py.in,v 1.151 2007/04/13 10:00:35 rockyb Exp $"""
 
 # This part contains user entry subroutines.
 
-_debugger_name = 'pydb'
+__title__ = 'pydb'
 
 # The name of the debugger we are currently going by.
-_version = '1.20'
+__version__ = '1.22'
 
 import inspect, os, re, sys, traceback
 
@@ -48,13 +49,17 @@ def process_options(p, debugger_name, pkg_version, option_list=None):
                          action="store_true", default=False, 
                          help="Show lines before executing them. " +
                          "This option also sets --batch")
+    optparser.add_option("-F", "--fntrace", dest="fntrace",
+                         action="store_true", default=False, 
+                         help="Show functions before executing them. " +
+                         "This option also sets --batch")
     optparser.add_option("--batch", dest="noninteractive",
                          action="store_true", default=False, 
                          help="Don't run interactive commands shell on "+
                          "stops.")
     optparser.add_option("--basename", dest="basename",
                          action="store_true", default=False, 
-                         help="Filenames strip off basename, (e.g.For testing)"
+                         help="Filenames strip off basename, (e.g. for regression tests)"
                          )
     optparser.add_option("-x", "--command", dest="command",
                          action="store", type='string', metavar='FILE',
@@ -129,17 +134,21 @@ def process_options(p, debugger_name, pkg_version, option_list=None):
 ##         target(opts.pid, opts, p)
 ##         sys.exit()
 
-    if opts.linetrace: opts.noninteractive = True
+    if opts.linetrace or opts.fntrace: opts.noninteractive = True
+    p.fntrace = opts.fntrace
     p.linetrace = opts.linetrace
     p.noninteractive = opts.noninteractive
 
-    # --nx or -n
+    # --nx or -n ?
     if not opts.noexecute:
-        # Read $HOME/.pydbrc and ./.pydbrc
+        # Read debugger startup file(s), e.g. $HOME/.pydbrc and ./.pydbrc
+        debugger_startup = ".%src" % debugger_name
         if 'HOME' in os.environ:
-            envHome = os.environ['HOME']
-            p.setup_source("%s.%src" % (envHome, debugger_name))
-        p.setup_source(".%src" % debugger_name);
+            debug_startup_home = os.path.join(os.environ['HOME'],
+                                              debugger_startup)
+            p.setup_source(debug_startup_home)
+            
+        p.setup_source(debugger_startup)
 
     if opts.cd:
         os.chdir(opts.cd)
@@ -247,15 +256,18 @@ def debugger(dbg_cmds=None, add_exception_hook=True, add_threaddbg=False,
         _pydb_trace.reset()
         _pydb_trace._program_sys_argv = list(sys.argv)
         _pydb_trace._sys_argv = list(_pydb_trace._program_sys_argv)
-        _pydb_trace._sys_argv[:0] = [_debugger_name]
+        _pydb_trace._sys_argv[:0] = [__title__]
         _pydb_trace.main_dirname = os.path.dirname(sys.argv[0])
 
     # We don't support "run" so we'll make "run" and "R" be "restart"
     _pydb_trace.do_R = _pydb_trace.do_run = _pydb_trace.do_restart
     _pydb_trace.curframe=inspect.currentframe()
 
+    # Dispatcher needs to field BdbQuit exception
+    _pydb_trace.field_BdbQuit   = True
+
     if _pydb_trace.running:
-        _pydb_trace.do_next(2)
+        _pydb_trace.do_next('2')
         return
     _pydb_trace.running = True
 
@@ -293,9 +305,9 @@ def exception_hook(type, value, tb, dbg_cmds=None, cmdfile=None):
 
 # print help
 def help():
-    doc_file='%s.doc' % _debugger_name
+    doc_file='%s.doc' % __title__
     for dirname in sys.path:
-        fullname = os.path.join(dirname, _debugger_name, doc_file)
+        fullname = os.path.join(dirname, __title__, doc_file)
         if os.path.exists(fullname):
             sts = os.system('${PAGER-more} '+fullname)
             if sts: print 'Pager exit status: %s' % str(sts)
@@ -324,23 +336,24 @@ def help():
 
 # Post-Mortem interface
 
-def pm(dbg_cmds=None, frameno=1):
+def pm(dbg_cmds=None, frameno=1, p=None):
 
     """Set up post-mortem debugging using the last traceback.  But if
     there is no traceback, we'll assume that sys.exec_info() contains
     what we want and frameno is the index location of where we want
     to start.
 
-    dbg_cmds is an optional list of debugger commands you want to run.
+    'dbg_cmds' is an optional list of debugger commands you want to run.
+    'p', is an optional pydb.Pdb object.
     """
 
     # frameno+1 because we are about to add one more level of call
     # in get_last_tb_or_frame_tb
     tb = fns.get_last_tb_or_frame_tb(frameno+1)
-    post_mortem(tb, dbg_cmds)
+    post_mortem(tb, dbg_cmds=dbg_cmds, p=p)
     return
 
-def post_mortem(t=None, dbg_cmds=None, cmdfile=None, frameno=1):
+def post_mortem(t=None, dbg_cmds=None, cmdfile=None, frameno=1, p=None):
     """Enter debugger read loop after your program has crashed.
 
     If no traceback parameter, t, is supplied, the last traceback and
@@ -348,18 +361,19 @@ def post_mortem(t=None, dbg_cmds=None, cmdfile=None, frameno=1):
     contains what we want and frameno is the index location of where
     we want to start.
 
-    dbg_cmds is a list of debugger commands you want to run.
+    'dbg_cmds' is an optional list of debugger commands you want to run.
 
-    cmdfile is an optional debugger command file you want to run
+    'cmdfile' is an optional debugger command file you want to run
     "source" on.
 
-    frameno specifies how many frames to ignore in the traceback.  The
+    'frameno' specifies how many frames to ignore in the traceback.  The
     default is 1 - we don't need the call to post_mortem. If you have
     wrapper functions that call this one, you may want to increase
     frameno.
     """
 
-    p = Pdb()
+    if p == None:
+        p = Pdb()
     p.reset()
     p.running = False
     re_bogus_file = re.compile("^<.+>$")
@@ -370,7 +384,7 @@ def post_mortem(t=None, dbg_cmds=None, cmdfile=None, frameno=1):
         t = fns.get_last_tb_or_frame_tb(frameno+1)
         if t is None:
             print "Can't find traceback for post_mortem " + \
-                  "in sys.last_traceback or sys.exec_info"
+                  "in sys.last_traceback or sys.exec_info()"
             return
 
     # t has least-recent traceback entry first. We want the most-recent
@@ -394,7 +408,7 @@ def post_mortem(t=None, dbg_cmds=None, cmdfile=None, frameno=1):
     if 0 == len(p._sys_argv):
         # Fake script invocation (restart) args since we don don't have any
         p._sys_argv = list(p._program_sys_argv)
-        p._sys_argv[:0] = [_debugger_name]
+        p._sys_argv[:0] = [__title__]
 
     if cmdfile is not None:
         p.do_source(cmdfile)
@@ -408,7 +422,11 @@ def post_mortem(t=None, dbg_cmds=None, cmdfile=None, frameno=1):
         except:
             pass
 
-        p.interaction(t.tb_frame, t)
+        # Possibly a bug in Python 2.5. Why f.f_lineno is
+        # not always equal to t.tb_lineno, I don't know.
+        f = t.tb_frame
+        if f and f.f_lineno != t.tb_lineno : f = f.f_back
+        p.interaction(f, t)
     except Restart:
         while True:
             sys.argv = list(p._program_sys_argv)
@@ -511,7 +529,7 @@ def runv(argv, p=None):
 
     This function may be useful inside Python shells.
 
-    Parapmeter p allows you to pass in your own Pdb object.
+    Parameter p allows you to pass in your own Pdb object.
     """
     ## FIXME: redo so we don't call main and don't have to save/restore
     ### sys.argv              
@@ -585,7 +603,7 @@ def main(p=None):
 
     if p is None:
         p = Pdb()
-    (opts, p) = process_options(p, _debugger_name, '1.20')
+    (opts, p) = process_options(p, __title__, __version__)
 
     # process_options has munged sys.argv to remove any options that
     # options that belong to this debugger. The original options to
@@ -604,9 +622,21 @@ def main(p=None):
 
         mainpyfile = p._program_sys_argv[0] # Get script filename.
         if not os.path.exists(mainpyfile):
-            print "pydb: Python script file '%s' does not exist" \
-                  % mainpyfile
-            sys.exit(1)
+            mainpyfile=fns.whence_file(mainpyfile)
+            if not os.path.exists(mainpyfile):
+                print "pydb: Python script file '%s' does not exist" \
+                      % mainpyfile
+                sys.exit(1)
+                return
+
+        # If mainpyfile is an optimized Python script try to find and
+        # use non-optimized alternative.
+        mainpyfile_noopt = fns.file_pyc2py(mainpyfile)
+        if mainpyfile != mainpyfile_noopt \
+               and os.path.exists(mainpyfile_noopt):
+            print "pydb: Compiled Python script given and we can't use that."
+            print "pydb: Substituting non-compiled name: %s" % mainpyfile_noopt
+            mainpyfile = mainpyfile_noopt
 
         # Replace pydb's dir with script's dir in front of
         # module search path.
@@ -622,8 +652,8 @@ def main(p=None):
         # right.
 
         try:
-            if p._program_sys_argv and p._program_sys_argv[0]:
-                p._runscript(p._program_sys_argv[0])
+            if p._program_sys_argv and mainpyfile:
+                p._runscript(mainpyfile)
                 if p.sig_received: 
                     p.sig_received = False
                     continue
@@ -633,7 +663,12 @@ def main(p=None):
 
             if p._user_requested_quit: break
             if p.noninteractive: break
-            p.msg("The program finished and will be restarted")
+            if p.fntrace: 
+               # fn tracing does something to not put a stop at the beginning.
+                p.msg("The program finished")
+                p.interaction(None, None)
+            else:
+                p.msg("The program finished and will be restarted")
         except Restart:
             if p._program_sys_argv:
                 sys.argv = list(p._program_sys_argv)
@@ -648,16 +683,23 @@ def main(p=None):
             p.running = False;
             if p.noninteractive or hasattr(p,'desired_thread'): break
         except:
-            traceback.print_exc()
+            traceback.print_exc(file=p.stdout)
             if p.noninteractive:
                 p.errmsg("Uncaught exception.")
                 break
             p.errmsg("Uncaught exception. Entering post mortem debugging")
-            p.errmsg( "Running 'c' or 'step' will restart the program")
+            if mainpyfile:
+                p.errmsg( "Running 'c' or 'step' will restart the program")
             p.running = False;
             t = sys.exc_info()[2]
-            while t.tb_next is not None:
-                t = t.tb_next
+
+            # I believe the commented code below discards information
+            # about where the exception was originally raised and just
+            # keeps us at the point where the exception was handled. I
+            # think though we want to this information.
+
+#             while t.tb_next is not None:
+#                 t = t.tb_next
             try:
                 p.interaction(t.tb_frame,t)
             except Restart:
@@ -667,7 +709,8 @@ def main(p=None):
                             " ".join(p._program_sys_argv[1:])))
             else:
                 p.msg("Post mortem debugger finished.")
-                p.msg(mainpyfile + " will be restarted")
+                if mainpyfile:
+                    p.msg(mainpyfile + " will be restarted")
         p.step_ignore = 0
 
     if p.stdout != sys.stdout:
