@@ -1,11 +1,12 @@
-"""$Id: pydbcmd.py,v 1.32 2006/10/13 22:45:10 rockyb Exp $
-A Python debugger command class.
+"""A Python debugger command class.
 
 Routines here have to do with parsing or processing commands, but are
 not the commands themselves which are in gdb.py.in.  Generally (but
 not always) they are not specific to pydb. They are sort of more
 oriented towards any gdb-like debugger. Also routines that need to be
-changed from cmd are here.  """
+changed from cmd are here.
+
+$Id: pydbcmd.py,v 1.42 2007/02/20 18:21:52 rockyb Exp $"""
 
 import cmd, linecache, sys, types
 from fns import *
@@ -24,15 +25,32 @@ class Cmd(cmd.Cmd):
         self._user_requested_quit = False
         self.aliases              = {}
         self.cmdtrace             = False
+        self.flush                = False        # flush after each write
         self.logging              = False
         self.logging_file         = "pydb.txt"
         self.logging_fileobj      = None         # file object from open()
         self.logging_overwrite    = False
         self.logging_redirect     = False
-        self.nohelp               = 'Undefined command: \"%s\". Try \"help\".'
+        self.nohelp               = 'Undefined command or invalid expression \"%s\".\nType \"help\" for a list of debugger commands.'
         self.prompt               = '(Pydb) '
         self.rcLines              = []
 
+    def print_source_line(self, lineno, line):
+        """Print out a source line of text , e.g. the second
+        line in:
+            (/tmp.py:2):  <module>
+            2 import sys,os
+            (Pydb)
+
+        We define this method
+        specifically so it can be customized for such applications
+        like ipython."""
+
+        # We don't use the filename normally. ipython and other applications
+        # however might.
+        self.msg_nocr('%d %s' % (lineno, line))
+        return
+
     def _runscript(self, filename):
         # When bdb sets tracing, a number of call and line events happens
         # BEFORE debugger even reaches user's code (and the exact sequence of
@@ -82,12 +100,38 @@ class Cmd(cmd.Cmd):
             else: exc_type_name = t.__name__
             self.errmsg('%s: %s' % (str(exc_type_name), str(v)))
 
-    ### This comes from cmd.py with self.stdout.write replaced by self.msg
+    ### This comes from cmd.py with self.stdout.write replaced by self.msg.
+    ### Also we extend to given help on an object name. The 
+    ### Docstring has been updated to reflect all of this.
     def do_help(self, arg):
-        """Without argument, print the list of available commands.
-        With a command name as argument, print help about that command
-        'help *cmd*' pipes the full documentation file to the $PAGER
-        'help exec gives help on the ! command"""
+        """help [command [subcommand]|expression]
+
+Without argument, print the list of available debugger commands.
+
+When an argument is given, it is first checked to see if it is command
+name. 'help exec' gives help on the ! command.
+
+With the argument is an expression or object name, you get the same
+help that you would get inside a Python shell running the built-in
+help() command.
+
+If the environment variable $PAGER is defined, the file is
+piped through that command.  You'll notice this only for long help
+output.
+
+Some commands like 'info', 'set', and 'show' can accept an
+additional subcommand to give help just about that particular
+subcommand. For example 'help info line' give help about the
+\code{info line} command.
+
+See also 'examine' an 'whatis'.
+        """
+
+        # We don't want to repeat the last help command. That makes
+        # not much sense and if give help that uses PAGER we may
+        # turn a quit CR into rerunning the help command.
+        self.lastcmd='' 
+
         if arg:
             first_arg = arg.split()[0]
             try:
@@ -99,8 +143,21 @@ class Cmd(cmd.Cmd):
                     self.msg("%s\n" % str(doc))
                     return
                 except AttributeError:
-                    pass
-                self.msg("%s\n" % str(self.nohelp % (first_arg,)))
+                    # If we have an object run site helper on that
+                    try:
+                        if not self.curframe:
+                            # ?? Should we have set up a dummy globals
+                            # to have persistence?
+                            value = eval(arg, None, None)
+                        else:
+                            value = eval(arg, self.curframe.f_globals,
+                                         self.curframe.f_locals)
+                        from site import _Helper
+                        h=_Helper()
+                        h.__call__(value)
+                    except:
+                       self.msg("%s\n" % str(self.nohelp % (first_arg,)))
+                       return
                 return
         else:
             names = self.get_names()
@@ -142,7 +199,7 @@ class Cmd(cmd.Cmd):
         To be compatible with onecmd will return 1 if we are to
         continue execution and None if not -- continue debugger
         commmand loop reading.  The remaining lines will still be in
-        self.rcLines.  """
+        self.rcLines."""
 
         if self.rcLines:
             # Make local copy because of recursion
@@ -151,8 +208,7 @@ class Cmd(cmd.Cmd):
             for line in rcLines:
                 self.rcLines = self.rcLines[1:]
                 line = line[:-1]
-                if self.cmdtrace: self.msg("+ %s" % line)
-                if len(line) > 0 and line[0] != '#':
+                if len(line) > 0:
                     # Some commands like step, continue,
                     # return return 1 to indicate execution
                     # should be continued. The return code
@@ -162,17 +218,46 @@ class Cmd(cmd.Cmd):
                     if self.onecmd(line) == 1:
                         return 1
 
+    def get_an_int(self, arg, errmsg=None, min_value=None, max_value=None):
+        """Another get_int() routine, this one simpler and less stylized
+        than get_int(). We eval arg return it as an integer value or
+        None if there was an error in parsing this.
+        """
+        if arg:
+            try:
+                # eval() is used so we will allow arithmetic expressions,
+                # variables etc.
+                ret_value = int(eval(arg)) 
+            except (SyntaxError, NameError, ValueError):
+                if errmsg:
+                    self.errmsg(errmsg)
+                else:
+                    self.errmsg('Expecting an integer, got: %s.' % str(arg))
+                return None
+
+        if min_value and ret_value < min_value:
+            self.errmsg('Expecting integer value to be at least %d, got: %d.' %
+                        (min_value, ret_value))
+            return None
+        elif max_value and ret_value > max_value:
+            self.errmsg('Expecting integer value to be at most %d, got: %d.' %
+                        (max_value, ret_value))
+            return None
+        return ret_value
+
     def get_int(self, arg, default=1, cmdname=None):
         """If arg is an int, use that otherwise take default."""
         if arg:
-            try: 
-                default = int(arg)
-            except ValueError:
+            try:
+                # eval() is used so we will allow arithmetic expressions,
+                # variables etc.
+                default = int(eval(arg)) 
+            except (SyntaxError, NameError, ValueError):
                 if cmdname:
-                    self.errmsg('%s command: Expecting an integer, got: %s' %
+                    self.errmsg('%s command: Expecting an integer, got: %s.' %
                                 (cmdname, str(arg)))
                 else:
-                    self.errmsg('Expecting an integer, got: %s' % str(arg))
+                    self.errmsg('Expecting an integer, got: %s.' % str(arg))
                 raise ValueError
         return default
 
@@ -192,25 +277,27 @@ class Cmd(cmd.Cmd):
             self.errmsg("Expecting 'on', 1, 'off', or 0. Got: %s." % str(arg))
         raise ValueError
 
-    def get_pos_int(self, arg, min=0, default = 1, cmdname=None):
+    def get_pos_int(self, arg, min_value=0, default=1, cmdname=None):
         """If no argument use the default If arg is a positive int at
-        least min, use that otherwise report an error."""
+        least min_value, use that otherwise report an error."""
         if arg:
             try: 
-                default = int(arg)
-                if default < min:
+                # eval() is used so we will allow arithmetic expressions,
+                # variables etc.
+                default = int(eval(arg))
+                if default < min_value:
                     if cmdname:
                         self.errmsg(('%s command: Expecting a positive ' +
-                                     'integer at least %d, got: %d') 
-                                    % (cmdname, min, default))
+                                     'integer at least %d, got: %d.') 
+                                    % (cmdname, min_value, default))
                     else: 
                         self.errmsg(('Expecting a positive ' +
                                      'integer at least %d, got: %d')
-                                    % (min, default))
+                                    % (min_value, default))
                     # Really should use something custom? 
                     raise ZeroDivisionError
                     
-            except ValueError:
+            except (SyntaxError, NameError, ValueError):
                 if cmdname:
                     self.errmsg(('%s command: Expecting a positive integer, '
                                  + "got: %s") % (cmdname, str(arg)))
@@ -282,11 +369,13 @@ class Cmd(cmd.Cmd):
         if self.logging:
             if self.logging_fileobj is not None:
                 print >> self.logging_fileobj, msg,
+                if self.flush: self.logging_fileobj.flush()
             do_print = not self.logging_redirect
         if do_print:
             if out is None:
                 out = self.stdout
             print >> out, msg,
+            if self.flush: out.flush()
 
     def precmd(self, line):
         """Method executed just before the command line line is
@@ -344,7 +433,10 @@ class Cmd(cmd.Cmd):
                 self.msg("")
 
             if print_line:
-                self.msg_nocr('+ %s' % linecache.getline(filename, lineno))
+                self.msg_nocr('+ ')
+            line=linecache.getline(filename, lineno)
+            if line and len(line.strip()) != 0:
+                self.print_source_line(lineno, line)
 
             # If we are stopped at an "exec" or print the next outer
             # location for that front-ends tracking source execution.
@@ -360,6 +452,7 @@ class Cmd(cmd.Cmd):
         prompt or in a breakpoint command list definition """
 
         if not self.commands_defining:
+            if self.cmdtrace: self.msg("+%s" % line)
             return cmd.Cmd.onecmd(self, line)
         else:
             return self.handle_command_def(line)
@@ -441,6 +534,7 @@ class Cmd(cmd.Cmd):
             # We've acknowledged a restart, which means that a new pdbserver
             # process is started, so we have to connect all over again.
             self._disconnect()
+            import time
             time.sleep(3.0)
             if not self.do_target(self.target_addr):
                 # We cannot trust these variables below to be in a
