@@ -1,6 +1,6 @@
 #!/usr/local/bin/python -t
-# $Id: import.py.in,v 1.2 2006/12/10 11:59:39 rockyb Exp $ -*- Python -*-
-"Unit test of program execution tracing using the Extended Python debugger "
+# $Id: import.py.in,v 1.3 2007/01/13 12:09:44 rockyb Exp $ -*- Python -*-
+"Unit test of 'import pydb'"
 import os, sys, unittest
 
 top_builddir = "../"
