@@ -1,6 +1,7 @@
 #!/usr/local/bin/python -t
-# $Id: test.py.in,v 1.35 2006/10/27 02:15:39 rockyb Exp $ -*- Python -*-
-"Unit test for the Extended Python debugger."
+# -*- Python -*-
+# $Id: test.py.in,v 1.38 2007/02/13 11:53:12 rockyb Exp $
+"Unit test for the Extended Python debugger's disassembler."
 import difflib, os, time, sys, unittest
 
 top_builddir = "../"
@@ -21,9 +22,11 @@ class PdbTest(pydb.Pdb):
         self.msgLines = []
         self.msg_last_nocr = False
         self.stack = self.curframe = self.botframe = None
+        return
 
     def errmsg(self, msg):
         self.errLines.append(msg)
+        return
 
     def msg(self, msg):
         if self.msg_last_nocr:
@@ -31,6 +34,7 @@ class PdbTest(pydb.Pdb):
         else:
             self.msgLines.append(msg)
         self.msg_last_nocr = False
+        return
 
     def msg_nocr(self, msg):
         if self.msg_last_nocr:
@@ -38,9 +42,11 @@ class PdbTest(pydb.Pdb):
         else:
             self.msgLines.append(msg)
         self.msg_last_nocr = True
+        return
 
     def msgReset(self):
         self.msgLines = []
+        return
 
 builddir     = "."
 if builddir[-1] != os.path.sep:
@@ -55,12 +61,17 @@ pydb_short   = "pydb.py"
 pydb_path    = os.path.join(pydir, pydb_short)
 
 def run_debugger(testname, pythonfile, args='', outfile=None,
-                 rightfile=None, pydb_opts='', truncate=None):
+                 rightfile=None, pydb_opts='', truncate=None,
+                 cmdfile=None):
     global srcdir, builddir, pydir
-    cmdfile     = os.path.join(srcdir, "%s.cmd" % testname)
+    if cmdfile is None:
+        cmdfile     = os.path.join(srcdir, "%s.cmd" % testname)
     outfile_opt = ''
     if outfile is None:
-        outfile     = "%s.out" % testname
+        test_dir    = ".."
+
+        outfile     = os.path.join("..", "test",
+                                   "%s.out" % testname)
         outfile_opt = '--output=%s ' % outfile
 
     if rightfile is None:
@@ -71,6 +82,7 @@ def run_debugger(testname, pythonfile, args='', outfile=None,
 
     if os.path.exists(outfile): os.unlink(outfile)
 
+    os.environ['PYTHONPATH']=os.pathsep.join(sys.path)
     cmd = "%s %s --command %s %s %s %s" % \
           (pydb_path, pydb_opts, cmdfile, outfile_opt, pythonfile, args)
     
@@ -123,17 +135,21 @@ class PdbTests(unittest.TestCase):
 
     def test_brkpt3(self):
         """Unit test breakpoint setting and display - part 3."""
+        cmdfile=os.path.join(builddir, 'brkpt3.cmd')
         result=run_debugger(testname='brkpt3',
                             pythonfile='%sgcd.py' % srcdir,
+                            rightfile=os.path.join(builddir, "brkpt3.right"),
                             args='3 5',
-                            truncate=-2)
+                            truncate=-2,
+                            cmdfile=cmdfile)
         self.assertEqual(True, result, "brkpt3")
         result=run_debugger(testname='brkpt3',
                             pythonfile='%sgcd.py' % srcdir,
                             args='3 5',
                             pydb_opts='--threading',
-                            rightfile=os.path.join(srcdir, "brkpt3t.right"),
-                            truncate=-2)
+                            rightfile=os.path.join(builddir, "brkpt3t.right"),
+                            truncate=-2,
+                            cmdfile=cmdfile)
         self.assertEqual(True, result, "brkpt3 --threading")
 
     def test_cmdparse(self):
@@ -203,12 +219,15 @@ class PdbTests(unittest.TestCase):
         result=run_debugger(testname='history',
                             pythonfile='%shanoi.py' % srcdir)
         self.assertEqual(True, result, "history")
+        if os.path.exists("history.hst"): os.unlink("history.hst")
 
     def test_logging(self):
         """Test 'set logging' and 'show logging' debugger commands."""
+        global top_builddir
+        logfile=os.path.join(top_builddir, "test", "logging.out")
         result=run_debugger(testname='logging',
                             pythonfile='%shanoi.py' % srcdir,
-                            outfile='logging.out')
+                            outfile=logfile)
         self.assertEqual(True, result, "logging")
 
     def test_no_script(self):
